unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, ExtCtrls, Buttons, Menus;

type
  TMainForm = class(TForm)
    leH: TLabeledEdit;
    leS: TLabeledEdit;
    PnlColor: TPanel;
    leR: TLabeledEdit;
    leG: TLabeledEdit;
    leWord: TLabeledEdit;
    SG: TStringGrid;
    Panel4: TPanel;
    Panel1: TPanel;
    pbHS: TPaintBox;
    Panel2: TPanel;
    pbV: TPaintBox;
    leV: TLabeledEdit;
    leB: TLabeledEdit;
    Label1: TLabel;
    CboxShowPallet: TCheckBox;
    PMenuText: TPopupMenu;
    PMenu01: TMenuItem;
    PMenu02: TMenuItem;
    PMenu03: TMenuItem;
    PMenu04: TMenuItem;
    PMenu05: TMenuItem;
    PMenu06: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure pbHSPaint(Sender: TObject);
    procedure pbVPaint(Sender: TObject);
    procedure pbHSMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbHSMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbHSMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure pbVMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure leHKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure leHChange(Sender: TObject);
    procedure SGDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PnlColorMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PnlColorMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SGSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure SGMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SGMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Label1Click(Sender: TObject);
    procedure CboxShowPalletClick(Sender: TObject);
    procedure PMenu06Click(Sender: TObject);
    procedure PMenu06DrawItem(Sender: TObject; ACanvas: TCanvas;
      ARect: TRect; Selected: Boolean);
    procedure PMenu06MeasureItem(Sender: TObject; ACanvas: TCanvas;
      var Width, Height: Integer);
  private
    HSVBmp : TBitmap;
    DrawBmp: TBitmap;
    FDragging: Boolean;

    HPos        : double;
    SPos        : double;
    VPos        : double;

    procedure   HSVMapToOther();
    procedure   HSVTextToOther();
    procedure   RGBTextToOther();
    procedure   WordTextToOther();
    procedure   TheColorToOther(AColor: TColor);

    procedure   DrawHSVColorMap();
    procedure   InitHTMLColorMap();
  public

  end;

var
  MainForm: TMainForm;

implementation

uses
    Info, iColor, ClipBrd;

const
    HEIGHT_SMALL = 136;
    HEIGHT_TALL  = 285;

{$R *.dfm}

//-----------------------------------------------------------------------------
//  ClientToScreen Rect
//-----------------------------------------------------------------------------
function    ClientRectToScreenRect(ASrcControl: TControl): TRect;
var
    W, H: integer;
    L, T: integer;
begin
    //  IWi̕ƍ
    with ASrcControl.ClientRect do
    begin
        W := Right - Left;
        H := Bottom - Top;
    end;
    //  IWi̍
    with ASrcControl.ClientOrigin do
    begin
        L := X;
        T := Y;
    end;
    Result := Bounds(L, T, W, H);
end;

//-----------------------------------------------------------------------------
//   MoveTo & LineTo
//-----------------------------------------------------------------------------
procedure   DrawLineDiff(C: TCanvas; const X, Y, DX, DY: integer);
begin
    C.MoveTo(X, Y);
    C.LineTo(X + DX, Y + DY);
end;

//-----------------------------------------------------------------------------
//  TEdit.OnChangeEĒlݒ
//-----------------------------------------------------------------------------
procedure   SetEditTextKillChange(AEdit: TLabeledEdit; const S: string);
var
    PreOnChange: TNotifyEvent;
begin
    PreOnChange := AEdit.OnChange;
    AEdit.OnChange := nil;
    AEdit.Text := S;
    AEdit.OnChange := PreOnChange;
end;

//-----------------------------------------------------------------------------
//  ݈ʒuȂɕ\
//-----------------------------------------------------------------------------
procedure   DrawCrossCursor(C: TCanvas; const X, Y: integer);
var
    PrePenW: integer;
begin
    PrePenW := C.Pen.Width;
    C.Pen.Width := 3;
    DrawLineDiff(C, X - 8, Y - 0,  4,  0);
    DrawLineDiff(C, X + 4, Y - 0,  4,  0);
    DrawLineDiff(C, X - 0, Y - 8,  0,  4);
    DrawLineDiff(C, X - 0, Y + 4,  0,  4);
    C.Pen.Width := PrePenW;
end;

//-----------------------------------------------------------------------------
//  HSVF}bv쐬
//-----------------------------------------------------------------------------
procedure   TMainForm.DrawHSVColorMap();
var
    R, G, B: Byte;
    XX, YY: integer;
    Line: PByteArray;
begin
    for YY := 0 to 100 - 1 do
    begin
        Line := PByteArray(HSVBmp.ScanLine[100 - 1 - YY]);
        for XX := 0 to 180 - 1 do
        begin
            HSVtoRGB(XX * 2, YY / 100, 1, R, G, B);
            Line[(XX) * 3 + 0] := B;
            Line[(XX) * 3 + 1] := G;
            Line[(XX) * 3 + 2] := R;
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  tH[RXgN^
//-----------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
begin
    SetFormCaption(Self);

    FDragging := false;

    HSVBmp := TBitmap.Create;
    HSVBmp.Height := 100;
    HSVBmp.Width  := 180;
    HSVBmp.PixelFormat := pf24bit;
    DrawHSVColorMap();

    DrawBmp := TBitmap.Create;
    DrawBmp.Height := 100;
    DrawBmp.Width  := 180;
    DrawBmp.PixelFormat := pf24bit;

    pbHS.ControlStyle := pbHS.ControlStyle + [csOpaque];
    pbV.ControlStyle := pbV.ControlStyle + [csOpaque];

    InitHTMLColorMap();

    //  ȃXy[X\
    Self.Height := HEIGHT_SMALL;
end;

//-----------------------------------------------------------------------------
//  ύX
//-----------------------------------------------------------------------------
procedure TMainForm.CboxShowPalletClick(Sender: TObject);
begin
    if (CboxShowPallet.Checked) then
    begin
        Self.Height := HEIGHT_TALL;
    end
    else
    begin
        Self.Height := HEIGHT_SMALL;
    end;
end;

//-----------------------------------------------------------------------------
//  tH[fXgN^
//-----------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    HSVBmp.Free;
    DrawBmp.Free;
end;

//-----------------------------------------------------------------------------
//  PaintBox:J[\ړ͈͂
//  IFXV
//-----------------------------------------------------------------------------
//  http://www5d.biglobe.ne.jp/~tomoya03/shtml/vbapi/ClipCursor.htm
//  Platform SDK
//-----------------------------------------------------------------------------
procedure TMainForm.pbHSMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    PB: TPaintBox;
    ClipRect: TRect;
begin
    PB := TPaintBox(Sender);
    //  J[\
    ClipRect := ClientRectToScreenRect(PB);
    ClipCursor(@ClipRect);
    //  hbOtOON
    FDragging := true;
    //  }EXMOVE
    PB.OnMouseMove(Sender, Shift, X, Y);
end;

//-----------------------------------------------------------------------------
//  PaintBox:J[\ړ͈͂tXN[
//-----------------------------------------------------------------------------
//  [Delphi:44951] Re: }EXIy[VȂ@ɂ
//-----------------------------------------------------------------------------
procedure TMainForm.pbHSMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    //  J[\
    ClipCursor(nil);
    //  hbOtOOFF
    FDragging := false;
end;

//-----------------------------------------------------------------------------
//  FEʓx}bv`
//-----------------------------------------------------------------------------
procedure TMainForm.pbHSPaint(Sender: TObject);
var
    PB: TPaintBox;
begin
    PB := TPaintBox(Sender);
    BitBlt(PB.Canvas.Handle, 0, 0, PB.Width, PB.Height,
            HSVBmp.Canvas.Handle, 0, 0, SRCCOPY);
    DrawCrossCursor(pbHS.Canvas, Trunc(HPos), Trunc(SPos));
end;

//-----------------------------------------------------------------------------
//  TStringGrid:J[\ړ͈͂
//-----------------------------------------------------------------------------
procedure TMainForm.SGMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    Grid: TStringGrid;
    ClipRect: TRect;
begin
    Grid := TStringGrid(Sender);
    //  J[\
    ClipRect := ClientRectToScreenRect(Grid);
    ClipCursor(@ClipRect);
end;

//-----------------------------------------------------------------------------
//  TStringGrid:J[\ړ͈͂tXN[
//-----------------------------------------------------------------------------
procedure TMainForm.SGMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    //  J[\
    ClipCursor(nil);
end;

//-----------------------------------------------------------------------------
//  x}bv`
//-----------------------------------------------------------------------------
procedure TMainForm.pbVPaint(Sender: TObject);
var
    PB: TPaintBox;
    H, S: double;
    R, G, B: Byte;
    XX, YY: integer;
    Line: PByteArray;
begin
    PB := TPaintBox(Sender);
    H := HPos * 2;
    S := (100 - SPos) / 100;

    for YY := 0 to 100 - 1 do
    begin
        Line := PByteArray(DrawBmp.ScanLine[100 - 1 - YY]);
        for XX := 0 to 24 - 1 do
        begin
            HSVtoRGB(H, S, YY / 100, R, G, B);
            Line[(XX) * 3 + 0] := B;
            Line[(XX) * 3 + 1] := G;
            Line[(XX) * 3 + 2] := R;
        end;
    end;

    BitBlt(PB.Canvas.Handle, 0, 0, PB.Width, PB.Height,
            DrawBmp.Canvas.Handle, 0, 0, SRCCOPY);
    DrawCrossCursor(pbV.Canvas, 11, Trunc(VPos));
end;

//-----------------------------------------------------------------------------
//  FEʓxI
//-----------------------------------------------------------------------------
procedure TMainForm.pbHSMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
    if FDragging then
    begin
        Hpos := X;
        SPos := Y;
        pbHS.Repaint();
        pbV.Repaint();
        HSVMapToOther();
    end;
end;

//-----------------------------------------------------------------------------
//  xI
//-----------------------------------------------------------------------------
procedure TMainForm.pbVMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
    if FDragging then
    begin
        VPos := Y;
        pbV.Repaint();
        HSVMapToOther();
    end;
end;

//-----------------------------------------------------------------------------
//  F}bv쎞̑XV
//-----------------------------------------------------------------------------
procedure   TMainForm.HSVMapToOther();
var
    H, S, V: double;
    R, G, B: Byte;
begin
    H := HPos * 2;
    S := (100 - SPos);
    V := (100 - VPos);
    HSVtoRGB(H, S / 100, V / 100, R, G, B);

    //  \
    PnlColor.Color := RGB(R, G, B);

    //  H, S, V
    SetEditTextKillChange(leH, Format('%.0f', [H]));
    SetEditTextKillChange(leS, Format('%.0f', [S]));
    SetEditTextKillChange(leV, Format('%.0f', [V]));
    //  R, G, B
    SetEditTextKillChange(leR, Format('%d', [R]));
    SetEditTextKillChange(leG, Format('%d', [G]));
    SetEditTextKillChange(leB, Format('%d', [B]));
    //  WORD
    SetEditTextKillChange(leWord, Format('%d', [PnlColor.Color]));
end;

//-----------------------------------------------------------------------------
//  HSVl쎞̑XV
//-----------------------------------------------------------------------------
procedure   TMainForm.HSVTextToOther();
var
    H, S, V: double;
    R, G, B: Byte;
begin
    H := StrToInt(leH.Text);
    S := StrToInt(leS.Text);
    V := StrToInt(leV.Text);

    HPos := H / 2;
    SPos := (100 - S);
    VPos := (100 - V);

    HSVtoRGB(H, S / 100, V / 100, R, G, B);

    //  \
    PnlColor.Color := RGB(R, G, B);

    //  R, G, B
    SetEditTextKillChange(leR, Format('%d', [R]));
    SetEditTextKillChange(leG, Format('%d', [G]));
    SetEditTextKillChange(leB, Format('%d', [B]));
    //  WORD
    SetEditTextKillChange(leWord, Format('%d', [PnlColor.Color]));
end;

//-----------------------------------------------------------------------------
//  RGBl쎞̑XV
//-----------------------------------------------------------------------------
procedure   TMainForm.RGBTextToOther();
var
    H, S, V: double;
    R, G, B: Byte;
begin
    R := StrToInt(leR.Text);
    G := StrToInt(leG.Text);
    B := StrToInt(leB.Text);

    RGBtoHSV(R, G, B, H, S, V);

    HPos := H / 2;
    SPos := 100 - 100 * S;
    VPos := 100 - 100 * V;

    //  \
    PnlColor.Color := RGB(R, G, B);

    //  H, S, V
    SetEditTextKillChange(leH, Format('%.0f', [H]));
    SetEditTextKillChange(leS, Format('%.0f', [S]));
    SetEditTextKillChange(leV, Format('%.0f', [V]));
    //  WORD
    SetEditTextKillChange(leWord, Format('%d', [PnlColor.Color]));
end;

//-----------------------------------------------------------------------------
//  WORDl쎞̑XV
//-----------------------------------------------------------------------------
procedure   TMainForm.WordTextToOther();
var
    W: TColor;
    H, S, V: double;
    R, G, B: Byte;
begin
    W := StrToInt(leWord.Text);
    R := GetRValue(W);
    G := GetGValue(W);
    B := GetBValue(W);

    RGBtoHSV(R, G, B, H, S, V);

    HPos := H / 2;
    SPos := 100 - 100 * S;
    VPos := 100 - 100 * V;

    //  \
    PnlColor.Color := RGB(R, G, B);

    //  R, G, B
    SetEditTextKillChange(leR, Format('%d', [R]));
    SetEditTextKillChange(leG, Format('%d', [G]));
    SetEditTextKillChange(leB, Format('%d', [B]));
    //  H, S, V
    SetEditTextKillChange(leH, Format('%.0f', [H]));
    SetEditTextKillChange(leS, Format('%.0f', [S]));
    SetEditTextKillChange(leV, Format('%.0f', [V]));
end;

//-----------------------------------------------------------------------------
//  FR{쎞̑XV
//-----------------------------------------------------------------------------
procedure   TMainForm.TheColorToOther(AColor: TColor);
var
    H, S, V: double;
    R, G, B: Byte;
begin
    R := GetRValue(AColor);
    G := GetGValue(AColor);
    B := GetBValue(AColor);

    RGBtoHSV(R, G, B, H, S, V);

    HPos := H / 2;
    SPos := 100 - 100 * S;
    VPos := 100 - 100 * V;

    //  \
    PnlColor.Color := AColor;

    //  R, G, B
    SetEditTextKillChange(leR, Format('%d', [R]));
    SetEditTextKillChange(leG, Format('%d', [G]));
    SetEditTextKillChange(leB, Format('%d', [B]));
    //  H, S, V
    SetEditTextKillChange(leH, Format('%.0f', [H]));
    SetEditTextKillChange(leS, Format('%.0f', [S]));
    SetEditTextKillChange(leV, Format('%.0f', [V]));
    //  WORD
    SetEditTextKillChange(leWord, Format('%d', [PnlColor.Color]));
end;

//-----------------------------------------------------------------------------
//  SpinEdit
//-----------------------------------------------------------------------------
//  http://www.comrade.co.jp/ftp/pg1/manual/VKeys.htm
//-----------------------------------------------------------------------------
procedure TMainForm.leHKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
    LE: TLabeledEdit;
    Val: integer;
begin
    LE := TLabeledEdit(Sender);

    case Key of

    VK_PRIOR:
        begin
            Val := StrToIntDef(LE.Text, 0) + 10;
            if (Val > LE.Tag) then Val := LE.Tag;
            LE.Text := IntToStr(Val);
        end;

    VK_NEXT:
        begin
            Val := StrToIntDef(LE.Text, 0) - 10;
            if (Val < 0) then Val := 0;
            LE.Text := IntToStr(Val);
        end;

    VK_UP:
        begin
            Val := StrToIntDef(LE.Text, 0) + 1;
            if (Val > LE.Tag) then Val := LE.Tag;
            LE.Text := IntToStr(Val);
        end;

    VK_DOWN:
        begin
            Val := StrToIntDef(LE.Text, 0) - 1;
            if (Val < 0) then Val := 0;
            LE.Text := IntToStr(Val);
        end;

    end;
end;

//-----------------------------------------------------------------------------
//  lڕύX
//-----------------------------------------------------------------------------
procedure TMainForm.leHChange(Sender: TObject);
var
    LE: TLabeledEdit;
    Val: integer;
begin
    LE := TLabeledEdit(Sender);
    Val := StrToIntDef(LE.Text, 0);
    if (Val < 0) then Val := 0 else if (Val > LE.Tag) then Val := LE.Tag;
    SetEditTextKillChange(LE, IntToStr(Val));

    if (Sender = leH) or (Sender = leS) or (Sender = leV) then
    begin
        HSVTextToOther();
    end
    else
    if (Sender = leR) or (Sender = leG) or (Sender = leB) then
    begin
        RGBTextToOther();
    end
    else
    if (Sender = leWORD) then
    begin
        WordTextToOther();
    end;

    pbHS.Repaint();
    pbV.Repaint();
end;

//-----------------------------------------------------------------------------
//  WEBF
//-----------------------------------------------------------------------------
const
    HTML_COLOR: array[0..5,0..5,0..5] of string
    = (
        (
            ('#000000','#000033','#000066','#000099','#0000cc','#0000ff'),
            ('#003300','#003333','#003366','#003399','#0033cc','#0033ff'),
            ('#006600','#006633','#006666','#006699','#0066cc','#0066ff'),
            ('#009900','#009933','#009966','#009999','#0099cc','#0099ff'),
            ('#00cc00','#00cc33','#00cc66','#00cc99','#00cccc','#00ccff'),
            ('#00ff00','#00ff33','#00ff66','#00ff99','#00ffcc','#00ffff')
        ),
        (
            ('#330000','#330033','#330066','#330099','#3300cc','#3300ff'),
            ('#333300','#333333','#333366','#333399','#3333cc','#3333ff'),
            ('#336600','#336633','#336666','#336699','#3366cc','#3366ff'),
            ('#339900','#339933','#339966','#339999','#3399cc','#3399ff'),
            ('#33cc00','#33cc33','#33cc66','#33cc99','#33cccc','#33ccff'),
            ('#33ff00','#33ff33','#33ff66','#33ff99','#33ffcc','#33ffff')
        ),
        (
            ('#660000','#660033','#660066','#660099','#6600cc','#6600ff'),
            ('#663300','#663333','#663366','#663399','#6633cc','#6633ff'),
            ('#666600','#666633','#666666','#666699','#6666cc','#6666ff'),
            ('#669900','#669933','#669966','#669999','#6699cc','#6699ff'),
            ('#66cc00','#66cc33','#66cc66','#66cc99','#66cccc','#66ccff'),
            ('#66ff00','#66ff33','#66ff66','#66ff99','#66ffcc','#66ffff')
        ),
        (
            ('#990000','#990033','#990066','#990099','#9900cc','#9900ff'),
            ('#993300','#993333','#993366','#993399','#9933cc','#9933ff'),
            ('#996600','#996633','#996666','#996699','#9966cc','#9966ff'),
            ('#999900','#999933','#999966','#999999','#9999cc','#9999ff'),
            ('#99cc00','#99cc33','#99cc66','#99cc99','#99cccc','#99ccff'),
            ('#99ff00','#99ff33','#99ff66','#99ff99','#99ffcc','#99ffff')
        ),
        (
            ('#cc0000','#cc0033','#cc0066','#cc0099','#cc00cc','#cc00ff'),
            ('#cc3300','#cc3333','#cc3366','#cc3399','#cc33cc','#cc33ff'),
            ('#cc6600','#cc6633','#cc6666','#cc6699','#cc66cc','#cc66ff'),
            ('#cc9900','#cc9933','#cc9966','#cc9999','#cc99cc','#cc99ff'),
            ('#cccc00','#cccc33','#cccc66','#cccc99','#cccccc','#ccccff'),
            ('#ccff00','#ccff33','#ccff66','#ccff99','#ccffcc','#ccffff')
        ),
        (
            ('#ff0000','#ff0033','#ff0066','#ff0099','#ff00cc','#ff00ff'),
            ('#ff3300','#ff3333','#ff3366','#ff3399','#ff33cc','#ff33ff'),
            ('#ff6600','#ff6633','#ff6666','#ff6699','#ff66cc','#ff66ff'),
            ('#ff9900','#ff9933','#ff9966','#ff9999','#ff99cc','#ff99ff'),
            ('#ffcc00','#ffcc33','#ffcc66','#ffcc99','#ffcccc','#ffccff'),
            ('#ffff00','#ffff33','#ffff66','#ffff99','#ffffcc','#ffffff')
        )
      );

//-----------------------------------------------------------------------------
//  HTML\̐Fϊ
//-----------------------------------------------------------------------------
function    HTMLToColor(const S: string; const Default: TColor = 0): TColor;
begin
    Result := StrToIntDef('$00' + Copy(S, 6, 2) + Copy(S, 4, 2) + Copy(S, 2, 2), Default);
end;

//-----------------------------------------------------------------------------
//  HTMLFObhɏ
//-----------------------------------------------------------------------------
procedure   TMainForm.InitHTMLColorMap();
var
    i, j: integer;
begin
    for i := 0 to SG.ColCount - 1 do
    begin
        for j := 0 to SG.RowCount - 1 do
        begin
            SG.Cells[i, j] := HTML_COLOR[i div 6, i mod 6, j];
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  WEBFObh`
//-----------------------------------------------------------------------------
procedure TMainForm.SGDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
    Grid: TStringGrid;
begin
    Grid := TStringGrid(Sender);

    if (gdFixed in State) then
    begin
        Grid.Canvas.Brush.Color := Grid.FixedColor;
        Grid.Canvas.FillRect(Rect);
    end
    else
    begin
        Grid.Canvas.Brush.Color := HTMLToColor(Grid.Cells[ACol, ARow]);
        Grid.Canvas.FillRect(Rect);
        if (gdFocused in State) or (gdSelected in State) then
        begin
            Grid.Canvas.Brush.Color := clBlack;
            Grid.Canvas.FrameRect(Rect);
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  WEBFObhI
//-----------------------------------------------------------------------------
procedure TMainForm.SGSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
var
    Grid: TStringGrid;
begin
    Grid := TStringGrid(Sender);
    TheColorToOther(HTMLToColor(Grid.Cells[ACol, ARow]));
    pbHS.Repaint();
    pbV.Repaint();
end;

//-----------------------------------------------------------------------------
//  ESCI
//-----------------------------------------------------------------------------
procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    case (Key) of

    //  ESCŕ
    VK_ESCAPE:
        begin
            Self.Close();
        end;

    //  ^[L[Ŏ͕̓i
    VK_RETURN:
        begin
            SendMessage(Self.Handle, WM_NEXTDLGCTL, 0, 0);
        end;

    end;
end;

//-----------------------------------------------------------------------------
//  pl̃}EX_E
//-----------------------------------------------------------------------------
procedure TMainForm.PnlColorMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    Panel: TPanel;
    Pt: TPoint;
var
    C: integer;
    R, G, B: Byte;
begin
    //  {^ŃLv`Jn
    if (Button = mbLeft) then
    begin
        Screen.Cursor := crCross;
    end
    //  E{^Ńj[\
    else if (Button = mbRight) then
    begin
        //  j[쐬
        C := ColorToRGB(StrToIntDef(leWord.Text, 0));
        R := GetRValue(C);
        G := GetGValue(C);
        B := GetBValue(C);

        //  낢
        PMenu01.Caption := Format('R=%3.3d,G=%3.3d,B=%3.3d', [R, G, B]);
        PMenu02.Caption := Format('COLOR=%s', [ColorToString(C)]);

        //  Delphi RGB
        PMenu03.Caption := Format('RGB(%3d,%3d,%3d);', [R, G, B]);

        //  Delphi TColor
        PMenu04.Caption := Format('TColor(%d);', [C]);

        //  BCB TColor
        PMenu05.Caption := Format('(TColor)%d;', [C]);

        //  HTML
        PMenu06.Caption := Format('#%s%s%s;', [IntToHex(R, 2), IntToHex(G, 2), IntToHex(B, 2)]);

        //  j[\
        Panel := TPanel(Sender);
        Pt := (ClientToScreen(Point(X, Y)));
        PMenuText.Popup(Panel.Left + Pt.X, Panel.Top + Pt.Y);
    end;
end;

//-----------------------------------------------------------------------------
//  pl̃}EXグ
//-----------------------------------------------------------------------------
procedure TMainForm.PnlColorMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    DC: HDC;
    Canvas: TCanvas;
    MPos: TPoint;
    Temp: TColor;
begin
    if (Button = mbLeft) then
    begin
        Screen.Cursor := crDefault;

        DC := GetDC(0);
        Canvas := TCanvas.Create;
        Canvas.Handle := DC;

        GetCursorPos(MPos);

        Temp := Canvas.Pixels[MPos.X, MPos.Y];
        TheColorToOther(Temp);
        pbHS.Repaint();
        pbV.Repaint();

        Canvas.Free;
        ReleaseDC(0, DC);
    end;
end;

//-----------------------------------------------------------------------------
//  o[W
//-----------------------------------------------------------------------------
procedure TMainForm.Label1Click(Sender: TObject);
begin
    OpenInfoForm();
end;

//-----------------------------------------------------------------------------
//  j[NbNŃRs[
//-----------------------------------------------------------------------------
procedure TMainForm.PMenu06Click(Sender: TObject);
var
    MenuItem: TMenuItem;
begin
    MenuItem := TMenuItem(Sender);
    Clipboard.AsText := StripHotKey(MenuItem.Caption);
end;

//-----------------------------------------------------------------------------
//  j[TCY
//-----------------------------------------------------------------------------
procedure TMainForm.PMenu06MeasureItem(Sender: TObject; ACanvas: TCanvas;
  var Width, Height: Integer);
var
    MenuItem: TMenuItem;
    Text: string;
    R: TRect;
    dtFormat: Cardinal;
    dtParams: TDrawTextParams;
    NewWidth: integer;
begin
    //  LXgȂ
    MenuItem := TMenuItem(Sender);
    Text := StripHotKey(MenuItem.Caption);

    //  wihԂ
    R := Rect(0, 0, 0, 0);
    ACanvas.Font.Assign(Self.Font);

    //  p[^ݒ
    dtFormat := DT_VCENTER + DT_SINGLELINE + DT_CALCRECT;
    FillChar(dtParams, SizeOf(dtParams), 0);
    dtParams.cbSize := SizeOf(dtParams);
    dtParams.iLeftMargin := 5;

    //  `V~[V
    DrawTextEx(ACanvas.Handle, PChar(Text), -1, R, dtFormat, @dtParams);

    //  TCY
    NewWidth := R.Right - R.Left;
    if (NewWidth > Width) then Width := NewWidth;
end;

//-----------------------------------------------------------------------------
//  j[`
//-----------------------------------------------------------------------------
procedure TMainForm.PMenu06DrawItem(Sender: TObject; ACanvas: TCanvas;
  ARect: TRect; Selected: Boolean);
var
    MenuItem: TMenuItem;
    Text: string;
    R: TRect;
    dtFormat: Cardinal;
    dtParams: TDrawTextParams;
begin
    //  LXgȂ
    MenuItem := TMenuItem(Sender);
    Text := StripHotKey(MenuItem.Caption);

    //  ͈͌
    R := ARect;
    ACanvas.Font.Assign(Self.Font);

    //  tHgF
    if (Selected) then
    begin
        ACanvas.Font.Color := clHighlightText;
        ACanvas.Brush.Color := clHighlight;
    end
    else
    begin
        ACanvas.Font.Color := clWindowText;
        ACanvas.Brush.Color := clBtnFace;
    end;

    //  wihԂ
    ACanvas.FillRect(R);

    //  p[^ݒ
    dtFormat := DT_VCENTER + DT_SINGLELINE;
    FillChar(dtParams, SizeOf(dtParams), 0);
    dtParams.cbSize := SizeOf(dtParams);
    dtParams.iLeftMargin := 5;

    //  `
    DrawTextEx(ACanvas.Handle, PChar(Text), -1, R, dtFormat, @dtParams);
end;

end.

