//  http://www.wakhok.ac.jp/~tetsuo/delphi/text/chap_f/top.html
//  F 摜|F̏
//  http://www.wakhok.ac.jp/~tetsuo/delphi/text/chap_f/pb_f02.html
//  F-2 HSV

unit iColor;

interface

uses
    Windows, SysUtils, Graphics;

procedure   RGBtoHSV(const R, G, B: Byte; var H, S, V: double); overload;
procedure   RGBtoHSV(const Color: TColor; var H, S, V: double); overload;

procedure   HSVtoRGB(const H, S, V: double; var R, G, B: Byte); overload;
procedure   HSVtoRGB(const H, S, V: double; var Color: TColor); overload;
function    HSVtoRGB(const H, S, V: double): TColor; overload;

implementation

uses
    Math;

// uRGB  HSV vZv
//  H   F hue        [0 - 360)
//  S   ʓx saturation [0 -   1]
//  V   x value      [0 -   1] 
procedure   RGBtoHSV(const R, G, B: Byte; var H, S, V: double);
var
    Min, Max, Delta: Byte;
    tH, tS, tV: double;
begin
    //  Fl̍őlEŏl擾
    Min := Math.Min(R, Math.Min(G, B));
    Max := Math.Max(R, Math.Max(G, B));
    Delta := Max - Min;

    //  x͍ől
    tV := Max;

    //  ʓx߂
    if (Max <> 0) then tS := (Max - Min) / Max else tS := 0;

    //  F߂
    tH := 0;
    if (tS <> 0) then
    begin
        //  Ԃő̎
        if (R = Max) then
        begin
            tH := 0 + (G - B) / Delta;
        end
        else if (G = Max) then
        begin
            tH := 2 + (B - R) / Delta;
        end
        else
        begin (* if (B = Max) then *)
            tH := 4 + (R - G) / Delta;
        end;
        tH := tH * 60;
        if (tH < 0) then tH := tH + 360;
    end;

    //  ߂
    H := tH;
    S := tS;
    V := tV / 255;
end;

// uRGB  HSV vZv - 2
procedure   RGBtoHSV(const Color: TColor; var H, S, V: double);
begin
    RGBtoHSV(GetRValue(Color), GetGValue(Color), GetBValue(Color), H, S, V);
end;

// uHSV  RGB vZv
//  H   F hue        [0 - 360)
//  S   ʓx saturation [0 -   1]
//  V   x value      [0 -   1]
procedure   HSVtoRGB(const H, S, V: double; var R, G, B: Byte);
var
    f, p, q, t: double;
    tH, tS, tV: double;
begin

    assert(H >= 0  , '0 >   H:' + FloatToStr(H));
    assert(H <= 360, 'H > 360:' + FloatToStr(H)) ;

    tH := H;    if (tH = 360) then tH := 0;
    tS := S;
    tV := V;

    //  ʓx = 0 ̂Ƃ͖x݂̂ŐF܂
    if (tS = 0) then
    begin
        R := Trunc(tV * 255);
        G := Trunc(tV * 255);
        B := Trunc(tV * 255);
    end
    else
    begin
        tH := tH / 60;
        f  := tH - Int(tH);
        p  := tV * (1 - tS);
        q  := tV * (1 - (tS * f)); 
        t  := tV * (1 - (tS * (1 - f)));

        case Trunc(tH) of
            0:
                begin
                    R := Trunc(tV * 255);
                    G := Trunc(t  * 255);
                    B := Trunc(p  * 255);
                end;
            1:
                begin
                    R := Trunc(q  * 255);
                    G := Trunc(tV * 255);
                    B := Trunc(p  * 255);
                end;
            2:
                begin
                    R := Trunc(p  * 255);
                    G := Trunc(tV * 255);
                    B := Trunc(t  * 255);
                end;
            3:
                begin
                    R := Trunc(p  * 255);
                    G := Trunc(q  * 255);
                    B := Trunc(tV * 255);
                end;
            4:
                begin
                    R := Trunc(t  * 255);
                    G := Trunc(p  * 255);
                    B := Trunc(tV * 255);
                end;
            5:
                begin
                    R := Trunc(tV * 255);
                    G := Trunc(p  * 255);
                    B := Trunc(q  * 255);
                end;
        end; 
    end;
end;

// uHSV  RGB vZv-2
procedure   HSVtoRGB(const H, S, V: double; var Color: TColor);
var
    R, G, B: Byte;
begin
    R := GetRValue(Color);
    G := GetGValue(Color);
    B := GetBValue(Color);
    HSVtoRGB(H, S, V, R, G, B);
    Color := RGB(R, G, B);
end;

// uHSV  RGB vZv-2
function    HSVtoRGB(const H, S, V: double): TColor;
begin
    HSVtoRGB(H, S, V, Result);
end;

end.
